"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// modules > 3rd party
const chalk_1 = __importDefault(require("chalk"));
// constants
const CWD = process.cwd() + '/';
const NODE_MODULES = 'node_modules/';
const INTERNAL = [
    '(_stream',
    '(events.js',
    '(internal/',
    '(vm.js',
];
const REGEX = new RegExp(`${CWD}([^:]+):(\\d+):(\\d+)`);
const REGEX_NODE_MODULES = /((?:@[^/]*\/)?[^/]*)([^:]*)/;
/*
 * Highlights a stack trace.
 *
 * {String} - String
 *
 * @returns A colored string of the stack trace
 */
function highlightStack(stack, { keepCWD, html } = {}) {
    return stack && stack
        .split('\n')
        .map((line) => {
        if (INTERNAL.some(str => line.includes(str))) {
            return chalk_1.default.grey(line);
        }
        return line.replace(REGEX, (_, path, line, column) => {
            if (path.includes(NODE_MODULES)) {
                const result = path.slice(NODE_MODULES.length).match(REGEX_NODE_MODULES);
                if (result) {
                    const [, module, file] = result;
                    if (html) {
                        return `${keepCWD ? CWD + NODE_MODULES : ''}${path.slice(parseInt(module, 10)).bold()}${file}:${line}:${column}`;
                    }
                    return `${chalk_1.default.grey(keepCWD ? CWD + NODE_MODULES : '')}${chalk_1.default.blue(module)}${chalk_1.default.grey(file)}:${chalk_1.default.grey.bold(line)}:${chalk_1.default.grey(column)}`;
                }
            }
            if (html) {
                return `${keepCWD ? CWD : ''}${path.bold()}:${line}:${column}`;
            }
            return `${chalk_1.default.grey(keepCWD ? CWD : '')}${chalk_1.default.yellow(path)}:${chalk_1.default.yellow.bold(line)}:${chalk_1.default.yellow(column)}`;
        });
    }).join('\n');
}
exports.default = highlightStack;
